import { Engine, EngineResult, Rule } from 'json-rules-engine';
import { GlobalRules, S3GetInput } from './types/UtilTypes';
import { S3 } from './libs/s3';
import { Logger } from '@aws-lambda-powertools/logger';
import { Axios } from './libs/axios';

const logger = new Logger({ serviceName: 'ApplyRules' });
const axios = new Axios();

async function getAccountInformation(accountId, urlDynamicFact) {
  try {
    const response = await axios.get(urlDynamicFact, {
      accountId: accountId,
    });

    return response;
  } catch (error) {
    logger.error(error);
    throw error;
  }
}

export async function applyRules(inputData: GlobalRules): Promise<any[]> {
  try {
    const bucketName = process.env.BUCKET_JSON_RULES;

    const param: S3GetInput = {
      Bucket: bucketName,
      Key: inputData.nameRule,
      VersionId: inputData?.versionNameRule,
    };

    const s3Data = await S3.getObject(param);
    const jsonRules: Rule[] = JSON.parse(s3Data).decisions;
    const engine = new Engine();
    let facts = inputData?.data;

    if (inputData?.factName) {
      const urlDynamicFact = process.env.URL_DYNAMIC_FACT;

      engine.addFact('account-information', async (_, almanac) => {
        try {
          const accountId = await almanac.factValue('accountId');
          const accountInfo = await getAccountInformation(
            accountId,
            urlDynamicFact
          );
          return accountInfo;
        } catch (error) {
          logger.error(error);
          throw error;
        }
      });

      facts = { accountId: inputData.factName };
    }

    jsonRules.forEach((rule) => {
      engine.addRule(rule);
    });

    const results: EngineResult = await engine.run(facts);

    return [results];
  } catch (error) {
    logger.error(error);
    throw error;
  }
}
